// This file is licensed under the terms of the CC-LGPL.
//
// Autor  : Rafael Angel Campos Vargas
// Correo : RofoelCompos@hotmail.com  
// Apdo. 964-1250 Escazu, San Jose, Costa Rica
// dedicado a Jesus y Maria...
//
// En este momento estoy creando un programa
// de animacion LibreN3D para FreeDOS con el compilador FreePascal
// falta mucho... 18 octubre 2016
//                
// Mis dibujos estan en unidades L (Lisa)
// 1 L equivale a 0.055063 pulgadas 
// definido como 0.001 para POVRAY
//      
// DTSG DuchaTermicaSalidaGravitatoria
// El archivo dibuja una ducha caliente para uso domestico, 
// es muy poco flexible pero incluye una macro para modificar
// texturas y unas pocas caracteristicas. Es compatible con POV-Ray 3.1.
//
// La macro principal tiene solo un parametro.   
// #macro DTSG_Ducha( FraccionPerilla )
//      FraccionPerilla         : Fraccion propia con signo, para desplazar la
//              perilla de control respecto a su posicion central en 0.0. 
//
// La siguiente macro permite mayor control de las caracteristicas.
// #macro DTSG_DuchaTermicaSalidaGravitatoria( FraccionPerilla,
//                AltoTotal, FraccionSuperior, DiametroSuperior,
//                FraccionInferior, DiametroInferior,
//                AnguloManguerita, DiametroTubo,
//                MaterialPrincipal, MaterialSuperior,
//                MaterialSalida )
//      FraccionPerilla         : Fraccion propia con signo, para desplazar la
//              perilla de control respecto a su posicion central en 0.0. 
//      AltoTotal               : Alto total ocupado por la ducha.
//              Valor por defecto 109*L.
//      FraccionSuperior        : Fraccion propia del total ocupado por la
//              seccion superior de la ducha con entrada de cables electricos.
//              Valor por defecto 0.270.
//      DiametroSuperior        : Diametro del cilindro imaginario de union 
//              del sector superior de la ducha respecto al sector medio.
//              Valor por defecto 58*L.
//      FraccionInferior        : Fraccion propia del total ocupado por la
//              seccion inferior de la ducha para salida del chorro de agua.
//              Valor por defecto 0.285.
//      DiametroInferior        : Diametro del cilindro imaginario de union 
//              del sector inferior de la ducha respecto al sector medio.
//              Valor por defecto 89*L.
//      AnguloManguerita        : Posicion angular de la salida para una manguerita
//              adicional a la ducha. El angulo se mide respecto a la posicion frontal.
//              Valor por defecto 105.
//      DiametroTubo            : Diametro del tubo principal para la entrada de agua.
//              Valor por defecto 13*L.
//      MaterialPrincipal       : Material principal para la ducha.
//              Valor por defecto DTSG_MaterialPrincipal.
//      MaterialSuperior        : Material a aplicar en la seccion superior de la ducha.
//              Valor por defecto DTSG_MaterialSuperior.
//      MaterialSalida          : Material a aplicar a la salida del chorro de agua.
//              Valor por defecto DTSG_MaterialSalida.
//      
// La ducha con sus valores por defecto se encuentra completamente contenida en:   
//      Punto Minimo    = <-47,  0,-48>*L
//      Punto Maximo    = < 47,109, 47>*L  
// El alto concuerda con el parametro AltoTotal, el diametro real sera ligeramente superior
// a DiametroInferior. 
// La salida de agua apunta hacia Y-, y las entradas de agua y cables hacia X-. El eje
// principal de la ducha se encuentra centrado sobre el eje Y, reposando directamente 
// encima del plano XZ.
//
// Estas son variables predefinidas o generadas en el
// interior de las macros que no deberian modificarse directamente.
// Si quiere cambiarlas, recurra a los modificadores.  
// #declare DTSG_DefaultMaterialPrincipal       
//      Material principal para la ducha.
// #declare DTSG_DefaultMaterialSuperior        
//      Material a aplicar a la seccion superior de la ducha.
// #declare DTSG_DefaultMaterialSalida        
//      Material a aplicar a la salida del chorro de agua.
//
// Los siguientes son los modificadores
// aplicables y redefinibles por el usuario (si se indica es default)   
// #declare DTSG_MaterialPrincipal              = material { ... }       
//      Material principal para la ducha.
// #declare DTSG_MaterialSuperior               = material { ... }
//      Material a aplicar a la seccion superior de la ducha.
// #declare DTSG_MaterialSalida                 = material { ... }
//      Material a aplicar a la salida del chorro de agua.
//        
// Para colocarlo simplemente
// #include "DTSG.inc" 
// material                     {
//   DTSG_DuchaTermicaSalidaGravitatoria( 0.0, 
//                0.109, 0.270, 0.058, 0.285, 0.089, 105, 0.013,
//                DTSG_MaterialPrincipal, DTSG_MaterialSuperior,
//                DTSG_MaterialSalida )  
// }
//                                 
// Ejemplo con la macro detallada, mismo objeto default
// #include "DTSG.inc"   
// object                       {
//   DTSG_PlanchaCuidadoMaterialCaliente( 0, off, 0.140, 0.007, 0.020, 2.86, 0.084, 0.025,
//        DTSG_MaterialPrincipal, DTSG_MaterialMetalico,
//        DTSG_MaterialMetalicoCaliente, DTSG_MaterialBaseAgarradera )  
// }
//        
// Con modificadores
// #include "DTSG.inc"
// #include "textures.inc"
// #declare DTSG_MaterialPrincipal   = material { textures { PinkAlabaster } }
// object        { DTSG_Ducha( 0.0 ) }
//
#include "colors.inc"         
#include "stones.inc"
#declare DTSG_L                         = 0.001;

#declare DTSG_DefaultMaterialPrincipal  =
  material                              {
    texture                             {  
      T_Grnt16     
      scale         10*DTSG_L
      normal                            {
        bumps
      }
    }
  }

#ifndef ( DTSG_MaterialPrincipal )
  #declare DTSG_MaterialPrincipal       = DTSG_DefaultMaterialPrincipal
#end

#declare DTSG_DefaultMaterialSuperior   =
  material                              {
    texture                             {  
      T_Stone36     
      scale         10*DTSG_L
      normal                            {
        bumps
      }
    }
  }

#ifndef ( DTSG_MaterialSuperior )
  #declare DTSG_MaterialSuperior        = DTSG_DefaultMaterialSuperior
#end

#declare DTSG_DefaultMaterialSalida     =
  material                              {
    texture                             {  
      T_Grnt16     
      scale         100*DTSG_L
      normal                            {
        bumps
      }
    }
  }

#ifndef ( DTSG_MaterialSalida )
  #declare DTSG_MaterialSalida          = DTSG_DefaultMaterialSalida
#end


#macro DTSG_DuchaTermicaSalidaGravitatoria( FraccionPerilla,
                AltoTotal, FraccionSuperior, DiametroSuperior,
                FraccionInferior, DiametroInferior,
                AnguloManguerita, DiametroTubo,
                MaterialPrincipal, MaterialSuperior,
                MaterialSalida )
  union                                 {
    // Sector Superior
    union                               {
      #local AnguloMaximoPerilla        = 10;
      #local FraccionCurvaturaPerilla   = 0.2;
      #local Perilla                    =
        union                           {
          box                           {
            <0,-0.5,-(1-FraccionCurvaturaPerilla)>,
            <1,+0.5,+(1-FraccionCurvaturaPerilla)>
          }
          difference                    {
            box                         {
              <0,-0.5,-1>,
              <FraccionCurvaturaPerilla,+0.5,1>
            }
            cylinder                    {
              -y,y,FraccionCurvaturaPerilla
              translate FraccionCurvaturaPerilla*x
              translate                +z
            }
            cylinder                    {
              -y,y,FraccionCurvaturaPerilla
              translate FraccionCurvaturaPerilla*x
              translate                -z
            }
          }
        }
      difference                        {
        union                           {
          #local QuiebreSuperior        =
                        0.025*FraccionSuperior*AltoTotal/2;
          cylinder                      {
            QuiebreSuperior*y, FraccionSuperior*AltoTotal*y/2, DiametroSuperior/2
          }
          cylinder                      {
            0, QuiebreSuperior*y, 0.9*DiametroSuperior/2
          }
          sphere                        {
            0, 1
            scale       < DiametroSuperior/2,
                          FraccionSuperior*AltoTotal/2, DiametroSuperior/2 >
            translate                   FraccionSuperior*AltoTotal*y/2
          }
        }
        difference                      {
          cylinder                      {
            -1.025*FraccionSuperior*AltoTotal/8*y,
            +1.025*FraccionSuperior*AltoTotal/8*y, 1.1*DiametroSuperior/2
          }
          cylinder                      {
            -1.25*FraccionSuperior*AltoTotal/8*y,
            +1.25*FraccionSuperior*AltoTotal/8*y, 0.95*DiametroSuperior/2
          }
          plane                         {
            x,0.75*DiametroSuperior/2
          }
          translate                     FraccionSuperior*AltoTotal*y/4
        }
      }
      // Perilla
      union                             {
        #local PosXPerilla              = 1.0125*DiametroSuperior/2;
        #local GruesoPerilla            = 0.275*FraccionSuperior*AltoTotal/2;
        #local RadioCilindroPerilla     =
                  sqrt( PosXPerilla*PosXPerilla+GruesoPerilla*GruesoPerilla );
        object                          {
          Perilla
          scale                 GruesoPerilla
          translate             +PosXPerilla*x
        }
        intersection                    {
          cylinder                      {
            -FraccionSuperior*AltoTotal/8*y,
            +FraccionSuperior*AltoTotal/8*y, RadioCilindroPerilla
          }
          plane                         {
            -x, -0.9*DiametroSuperior/2
          }
        }          
        material                        {
          MaterialPrincipal
        }
        rotate                  AnguloMaximoPerilla*FraccionPerilla*y
        translate               +FraccionSuperior*AltoTotal/4*y
      }
      // END Perilla
      // Entrada Cables
      difference                        {
        union                           {
          superellipsoid                {
            <0.2,0.2>
            scale       <1.2*DiametroSuperior/2,FraccionSuperior*AltoTotal/2,
                          0.25*DiametroSuperior/2>
          }
          superellipsoid                {
            <0.2,0.2>
            scale       <1.2*DiametroSuperior/2,FraccionSuperior*AltoTotal/4,
                          0.30*DiametroSuperior/2>
          }
          cylinder                      {
            #local RadioC               =
               min( 0.20*DiametroSuperior/2, 0.2*FraccionSuperior*AltoTotal );
            0, -( 0.7*DiametroInferior+0.3*DiametroSuperior )/2*x, RadioC
            translate                   RadioC*y
          }
        }
        plane                           {
          y, 0
        }
        plane                           {
          -x, 0
        }
        cylinder                      {
          0, -2*( 0.7*DiametroInferior+0.3*DiametroSuperior )/2*x,
          0.8*RadioC
          translate                   RadioC*y
        }    
        material                        { 
          MaterialPrincipal
        }
      }             
      // END Entrada Cables
      material                          {               
        MaterialSuperior
      }
      translate                 ( 1-FraccionSuperior )*AltoTotal*y
    }
    // END Sector Superior
    // Sector Medio
    union                               {
      #local AltoTotalSor               =
                        (1-FraccionSuperior-FraccionInferior)*AltoTotal;
      #local QuiebreSor                 = 0.0125*AltoTotalSor;
      cylinder                          {
        0, QuiebreSor*y, 0.95*DiametroInferior/2
      }
      sor                               {
        5,
        <DiametroInferior/2,-AltoTotalSor>,
        <DiametroInferior/2,QuiebreSor>,
        <9*DiametroSuperior/15,8*AltoTotalSor/20>,
        <DiametroSuperior/2,AltoTotalSor>,
        <3*DiametroSuperior/4,2*AltoTotalSor>
        sturm
      }
      // Tubo
      difference                        {
        #local MenorDiametro            =
                        min( DiametroInferior, DiametroSuperior );
        #local MayorDiametro            =
                        max( DiametroInferior, DiametroSuperior );
        union                           {
          cylinder                          {
            -1.1*MenorDiametro/2*x,
            ( -1.05*MayorDiametro-1.1*MenorDiametro )/4*x,
            DiametroTubo/2
          }
          cylinder                          {
            -1.1*MenorDiametro/2*x, -1.05*MayorDiametro/2*x,
            0.95*DiametroTubo/2
          }
          cylinder                          {
            -1.1*MenorDiametro/2*x, -1.05*MenorDiametro/2*x,
            1.25*DiametroTubo/2
          }
          cylinder                          {
            -1.1*MenorDiametro/2*x, 0*x,
            0.85*DiametroTubo/2
          }
          cylinder                      {
            -0.3*( 1-FraccionInferior-FraccionSuperior )*AltoTotal*y,
            +0.3*( 1-FraccionInferior-FraccionSuperior )*AltoTotal*y
            +0.25*FraccionSuperior*AltoTotal*y,
            0.05*MenorDiametro
            translate           -DiametroSuperior/2*x
          }
          box                           {
            <-1.1*MenorDiametro/2,-0.05*DiametroTubo,+DiametroTubo>,
            <0,+0.05*DiametroTubo,-DiametroTubo>
          }
          #local Pos0                   =
                        ( -1.05*MayorDiametro-1.1*MenorDiametro )/4;
          #local Distancia              =
                        ( +1.05*MayorDiametro-1.1*MenorDiametro )/4;
          #local TotalRoscas            = 6;
          #local D                      = 0;
          #while ( D < TotalRoscas  )
            cylinder                    {
              0, -Distancia/( 2.5*TotalRoscas )*x, DiametroTubo/2
              translate  Pos0*x-( D+0.5 )/TotalRoscas*Distancia*x
            }
            #local D                    = D+1;
          #end
        }
        cylinder                        {
          0, -MayorDiametro*x, 0.80*DiametroTubo/2
        }
        translate     0.7*( 1-FraccionInferior-FraccionSuperior )*AltoTotal*y
      }
      // END Tubo
      translate         FraccionInferior*AltoTotal*y
    }
    // END Sector Medio
    // Sector Inferior
    union                               {
      difference                        {
        sphere                          {
          0, 1
        }
        #local DiametroCirculito        = 2*pi/60;
        #local RR                       = 0;
        #while ( RR < 3 )
          #switch ( RR )
            #case ( 0 )
              #local RPlano             = 1/3;
              #local D0                 = 0;
              #local TCirculitos        = 12;
            #break
            #case ( 1 )
              #local RPlano             = 13/24;
              #local D0                 = 0;
              #local TCirculitos        = 24;
            #break
            #case ( 2 )
              #local RPlano             = 3/4;
              #local D0                 = 2;
              #local TCirculitos        = 32;
            #break
          #end
          #local HPlano                 = 1-sqrt( 1-RPlano*RPlano );
          #local D                      = D0;
          #while ( D < TCirculitos )
            difference                  {
              cylinder                  {
                -0.2*y, 0.2*y, DiametroCirculito/2
              }
              difference                {
                cylinder                {
                  0.1*y, 0.21*y, DiametroCirculito/4
                }
                cylinder                {
                  0.0*y, 0.22*y, 0.75*DiametroCirculito/4
                }
              }
              translate                 -y+RPlano*x+HPlano*y
              rotate            ( D-0.5 )/TCirculitos*360*y
              rotate            +AnguloManguerita*y
            }
            #local D                    = D+1;
          #end  // #while ( D < TotalCirculitos )
          #local RR                     = RR+1;
        #end  // #while ( RR < 3 )
        scale         < DiametroInferior/2,
                        FraccionInferior*AltoTotal/2. DiametroInferior/2 >
        translate                     FraccionInferior*AltoTotal*y/2
      }
      difference                        {
        cylinder                        {
          13/20*x*DiametroInferior/2, 1.1*x*DiametroInferior/2,
          0.1*FraccionInferior*AltoTotal
          translate     0.05*FraccionInferior*AltoTotal*y
        }
        cylinder                        {
          0, 1.2*x*DiametroInferior/2,
          0.09*FraccionInferior*AltoTotal
          translate     0.05*FraccionInferior*AltoTotal*y
        }      
        material                        {     
          MaterialPrincipal
        }
        translate       ( 1-sqrt( 1-9/16 ) )*y*FraccionInferior*AltoTotal*y/2
        rotate          AnguloManguerita*y
      }
      torus                             {
        #local CompresionTorus          = 1/3;
        DiametroInferior/2, CompresionTorus*FraccionInferior*AltoTotal/4
        scale   <1,1/CompresionTorus,1>
        translate                       3*FraccionInferior*AltoTotal*y/4
      }     
      material                          {    
        MaterialSalida
      }
    }
    // END Sector Inferior
    material                            {
      MaterialPrincipal
    }
  }
#end

#macro DTSG_Ducha( FraccionPerilla )
  object                                {
    #local L                            = DTSG_L;
    #local AltoTotal                    = 109*L;
    #local FraccionSuperior             = 0.270;
    #local DiametroSuperior             = 58*L;
    #local FraccionInferior             = 0.285;
    #local DiametroInferior             = 89*L;
    #local AnguloManguerita             = 105;
    #local DiametroTubo                 = 13*L;
    DTSG_DuchaTermicaSalidaGravitatoria( FraccionPerilla,
                AltoTotal, FraccionSuperior, DiametroSuperior,
                FraccionInferior, DiametroInferior,
                AnguloManguerita, DiametroTubo,
                DTSG_MaterialPrincipal,
                DTSG_MaterialSuperior,
                DTSG_MaterialSalida )
  }
#end  // #macro DTSG_Ducha()
