// This file is licensed under the terms of the CC-LGPL.
//
// Autor  : Rafael Angel Campos Vargas
// Correo : RofoelCompos@hotmail.com  
// Apdo. 964-1250 Escazu, San Jose, Costa Rica
// dedicado a Jesus y Maria...
//
// En este momento estoy creando un programa
// de animacion LibreN3D para FreeDOS con el compilador FreePascal
// falta mucho... 8 noviembre 2016
//                
// Mis dibujos estan en unidades L (Lisa)
// 1 L equivale a 0.055063 pulgadas 
// definido como 0.001 para POVRAY
//     
// GDHD GuillotinaDiezHojasDelgadas
// El archivo dibuja una guillotina para papel.
// Es muy poco flexible. Es compatible con POVRay 3.1.
//
// La macro principal tiene solo un parametro. 
// #macro GDHD_Guillotina( FraccionAbrir ) 
//      FraccionAbrir           : Fraccion propia para abrir la guillotina.
//              Use 0.0 para cerrado y 1.0 brazo extendido.
//
// La siguiente macro permite mayor versatilidad.
// #macro GDHD_GuillotinaDiezHojasDelgadas( FraccionAbrir,
//                FrenteFundamental, AltoFundamental, ProfundidadFundamental,
//                GruesoPalanca, LargoMango, GruesoFilo, AnchoHojaMetal, Cuadriculado,
//                MaterialPrincipal, MaterialMango,
//                MaterialMetal, MaterialFilo, MaterialReglaCentral )
//      FraccionAbrir           : Fraccion propia para abrir la guillotina.
//              Use 0.0 para cerrado y 1.0 brazo extendido.
//      FrenteFundamental       : Referencia para el frente X del aparato.
//              Valor por defecto 316*L.
//      AltoFundamental         : Referencia para el alto Y del aparato.
//              Valor por defecto 23*L.
//      ProfundidadFundamental  : Referencia para la profundidad Z del aparato.
//              Valor por defecto 227*L.
//      GruesoPalanca           : Grueso de la palanca de la guillotina.
//              Valor por defecto 24*L.
//      LargoMango              : Largo del mango para sujetar la palanca.
//              Valor por defecto 89*L.
//      GruesoFilo              : Grueso de las laminillas de metal.
//              Valor por defecto 2*L.
//      AnchoHojaMetal          : Ancho de la lamina metalica lateral.
//              Valor por defecto 15*L.
//      Cuadriculado            : Separacion del cuadriculado.
//              Valor por defecto 10*L.
//      MaterialPrincipal       : Material principal de la guillotina.
//              Valor por defecto GDHD_MaterialPrincipal.
//      MaterialMango           : Material del mango para la palanca.
//              Valor por defecto GDHD_MAterialMango.
//      MaterialMetal           : Material para hojas metalicas.
//              Valor por defecto GDHD_MaterialMetal.
//      MaterialFilo            : Material para simular el filo de la guillotina.
//              Valor por defecto GDHD_MaterialFilo.
//      MaterialReglaCentral    : Material para la reglilla central.
//              Valor por defecto GDHD_MaterialReglaCentral.
//      
// La guillotina cerrada con parametro 0.0 se encuentra completamente contenido
// en las siguientes coordenadas:
//      Minimo  = <-165,   0,-230 >*L
//      Maximo  = < 255,  50,  30 >*L     
// Com parametro 1.0: 
//      Minimo  = <-165,   0,-230 >*L
//      Maximo  = < 160, 325,  30 >*L 
// La palanca abre hacia X+, gravedad Y-. El plano YZ separa la palanca de la base.
// La guillotina descansa enteramente sobre el plano XZ.
//
// Estas son variables predefinidas o generadas en el
// interna de las macros que no deberian modificarse directamente.
// Si quiere cambiarlas, recurra a los modificadores.  
// #declare GDHD_DefaultMaterialPrincipal       
//      Material principal de la guillotina.
// #declare GDHD_DefaultMaterialMango           
//      Material del mango para la palanca.
// #declare GDHD_DefaultMaterialMetal           
//      Material para hojas metalicas.
// #declare GDHD_DefaultMaterialFilo           
//      Material para simular el filo de la guillotina.
// #declare GDHD_DefaultMaterialReglaCentral    
//      Material para la reglilla central.
//
// Los siguientes son los modificadores
// aplicables y redefinibles por el usuario (si se indica es default)   
// #declare GDHD_ColorMarcas                = White;
//      Color de las marcas indicadoras del lateral.
// #declare GDHD_MaterialPrincipal          = material { ... }
//      Material principal de la guillotina.
// #declare GDHD_MaterialMango              = material { ... }
//      Material del mango para la palanca.
// #declare GDHD_MaterialMetal              = material { ... }
//      Material para hojas metalicas.
// #declare GDHD_MaterialFilo               = material { ... }
//      Material para simular el filo de la guillotina.
// #declare GDHD_MaterialReglaCentral       = material { ... }
//      Material para la reglilla central.
//        
// Para colocarlo simplemente
// #include "GDHD.inc" 
// object { GDHD_Guillotina(0.0) }
//                                  
// Ejemplo con la macro de texturas, mismo objeto default
// #include "GDHD.inc"   
// object                       {
//   GDHD_GuillotinaDiezHojasDelgadas( 0.0, 
//        0.316, 0.023, 0.227, 0.024, 0.089, 0.002, 0.015, 0.010,
//        GDHD_MaterialPrincipal, GDHD_MaterialMango,
//        GDHD_MaterialMetal, GDHD_MaterialFilo,
//        GDHD_MaterialReglaCentral )
// }
//        
// Con modificadores
// #include "GDHD.inc"
// #include "textures.inc"
// #declare GDHD_TexturaPrincipal   = material { textire { PinkAlabaster } }
// object        { GDHD_Guillotina(0.0) }
// 

#include "stones.inc"
#declare GDHD_L                         = 0.001;

#declare GDHD_ColorMarcas               = White;

#declare GDHD_DefaultMaterialPrincipal  =
  material                              {
    texture                             {
      T_Grnt19
    }
  }

#ifndef ( GDHD_MaterialPrincipal )
  #declare GDHD_MaterialPrincipal       = GDHD_DefaultMaterialPrincipal
#end

#declare GDHD_DefaultMaterialMango      =
  material                              {
    texture                             {
      pigment                           {
        ForestGreen
      }
      normal                            {
        bumps
      }
    }
  }

#ifndef ( GDHD_MaterialMango )
  #declare GDHD_MaterialMango           = GDHD_DefaultMaterialMango
#end

#declare GDHD_DefaultMaterialMetal      =
  material                              {
    texture                                     {
      pigment                                   {
        color Silver
      }
      finish                                    {
        phong 0.2
      }
    }
  }

#ifndef ( GDHD_MaterialMetal )
  #declare GDHD_MaterialMetal           = GDHD_DefaultMaterialMetal
#end

#declare GDHD_DefaultMaterialFilo       =
  material                              {
    texture                                     {
      pigment                                   {
        color Silver
      }
      normal                            {
        radial
        frequency  2000
        sine_wave
        rotate                          90*x
      }
      finish                                    {
        phong 0.2
      }
    }
  }

#ifndef ( GDHD_MaterialFilo )
  #declare GDHD_MaterialFilo            = GDHD_DefaultMaterialFilo
#end

#declare GDHD_DefaultMaterialReglaCentral               =
  material                              {
    texture                                     {
      pigment                                   {
        gradient z
        scale           2*GDHD_L
        pigment_map                     {
          [ 0.00 Gray20 ]
          [ 0.95 Silver ]
          [ 0.95 White ]
          [ 1.0  White ]
        }
      }
      normal                            {
        bumps
      }
      finish                                    {
        phong 0.2
      }
    }
  }

#ifndef ( GDHD_MaterialReglaCentral )
  #declare GDHD_MaterialReglaCentral    = GDHD_DefaultMaterialReglaCentral
#end

#macro GDHD_GuillotinaDiezHojasDelgadas( FraccionAbrir,
                FrenteFundamental, AltoFundamental, ProfundidadFundamental,
                GruesoPalanca, LargoMango, GruesoFilo, AnchoHojaMetal,
                Cuadriculado,
                MaterialPrincipal, MaterialMango,
                MaterialMetal, MaterialFilo, MaterialReglaCentral )
  union                                 {
    #local AnguloMaximo                 = 45;
    #local FraccionPlana                = 285/316;
    #local FraccionDecorado             =  41/316;
    #local AltoMax                      = 38/23*AltoFundamental;
    union                               {
      difference                        {
        superellipsoid                  {
          <0.1,0.1>
          scale         <FrenteFundamental/2,
                         2.0*AltoFundamental,ProfundidadFundamental>
        }
        plane                           {
          y, 0
        }
        plane                           {
          x, -( FraccionPlana-0.5 )*FrenteFundamental*1.01
        }
        plane                           {
          -y, -AltoFundamental
        }
        plane                           {
          -z, 0
        }
        box                             {
          < -FrenteFundamental,  AltoFundamental, -AnchoHojaMetal >,
          < +0.47*FrenteFundamental, -GruesoFilo, GruesoPalanca >
          translate                     AltoFundamental*y
        }
        box                             {
          < -FrenteFundamental, -AltoFundamental, -0.1*GruesoPalanca >,
          < +0.49*FrenteFundamental, 2*AltoFundamental, GruesoPalanca >
        }
        // Regla Central por diferencia
        box                             {
          < 0,  AltoFundamental, -0.95*ProfundidadFundamental >,
          < -0.1*FrenteFundamental, -GruesoFilo, 0 >
          translate                     AltoFundamental*y
        }
        // Cuadriculado por diferencia
        #local TotalCuadriculaFrente            =
                   div( FraccionPlana*FrenteFundamental, Cuadriculado )-1;
        #local TotalCuadriculaProfundidad       =
                   div( ProfundidadFundamental, Cuadriculado )-1;
        #local GruesoLinea              = 0.05*Cuadriculado;
        #local C                        = 1;
        #while ( C < TotalCuadriculaFrente )
          box                           {
            < -GruesoLinea/2, -2*GruesoFilo, 0 >,
            < +GruesoLinea/2, +GruesoFilo,
              -( TotalCuadriculaProfundidad-1 )*Cuadriculado >
            translate           ( -( FraccionPlana-0.5 )*FrenteFundamental
                                        +C*Cuadriculado )*x+AltoFundamental*y
          }
          #local C                      = C+1;
        #end
        #local C                        = 1;
        #while ( C < TotalCuadriculaProfundidad )
          box                           {
            < -( FraccionPlana-0.5 )*FrenteFundamental+Cuadriculado,
              -2*GruesoFilo, -GruesoLinea/2 >,
            < -( FraccionPlana-0.5 )*FrenteFundamental
              +( TotalCuadriculaFrente-1 )*Cuadriculado, +GruesoFilo,
              +GruesoLinea/2 >
            translate           +AltoFundamental*y-C*Cuadriculado*z
          }
          #local C                      = C+1;
        #end
        // END Cuadriculado por diferencia
      }
      difference                        {
        sor                             {
          5,
          < ( FraccionPlana+FraccionDecorado-0.5 )*FrenteFundamental,
                -ProfundidadFundamental >,
          < ( FraccionPlana+FraccionDecorado-0.5 )*FrenteFundamental, 0 >,
          < ( FraccionPlana/4+0.25 )*FrenteFundamental, 0.5*ProfundidadFundamental >,
          < ( FraccionPlana/4+0.25 )*FrenteFundamental, ProfundidadFundamental >,
          < ( FraccionPlana/4+0.25 )*FrenteFundamental, 2*ProfundidadFundamental >
          rotate                        -90*x
          scale                 <1,8.35*AltoFundamental/FrenteFundamental,1>
        }
        plane                           {
          -x, ( FraccionPlana-0.5 )*FrenteFundamental
        }
        plane                           {
          y, 0
        }
        plane                           {
          #local AnguloDecorado         =
                        degrees( atan2( AltoMax,
                ( FraccionPlana+FraccionDecorado-0.5 )*FrenteFundamental ) );
          -y, 0
          rotate                        -AnguloDecorado*z
        }
        difference                      {
          superellipsoid                  {
            <0.1,0.1>
            scale         <FrenteFundamental/2,
                           2.0*AltoFundamental,ProfundidadFundamental>
            inverse
          }
          plane                         {
            -z, 0.35*ProfundidadFundamental
          }
        }
      }
      // Marcas Borde Curvo
      #local XMarcas                    =
                ( FraccionPlana-0.5 )*FrenteFundamental;
      #local HMarcas                    = tan( radians( AnguloDecorado ) )*
                ( FraccionPlana-0.5 )*FrenteFundamental;
      #local RMarcas                    =
                       sqrt( XMarcas*XMarcas+HMarcas*HMarcas );
      union                             {
        #local C                        = 1;
        #while ( C < TotalCuadriculaProfundidad )
          #local C                      = C+1;
          cylinder                      {
            0, -Cuadriculado*x, 0.05*Cuadriculado
            scale       <1,0.05,1>
            translate                   -RMarcas*x
            rotate                      -AnguloDecorado*z
            translate                   -C*Cuadriculado*z
            pigment { color GDHD_ColorMarcas }
          }
        #end  // #while ( C < TotalCuadriculaProfundidad )
      }
      // END Marcas Borde Curvo
      // Hoja Metalica Borde
      difference                        {
        box                             {
          < -( FraccionPlana-0.5 )*FrenteFundamental, 0, -AnchoHojaMetal >,
          < +0.47*FrenteFundamental, -GruesoFilo, 0 >
          translate                     AltoFundamental*y
        }
        #local C                        = -1;
        #while ( C < 2 )
          torus                           {
            AnchoHojaMetal/4, GruesoFilo/5
            scale                 <1,5,1>
            translate                     -AnchoHojaMetal*z/2
            translate       AltoFundamental*y+C*FrenteFundamental*x/4
          }
          #local C                      = C+1;
        #end  // #while ( C < 2 )
        material                        {
          MaterialMetal
        }
      }
      // END Hoja Metalica Borde
      // Regla Central
      box                             {
        < 0,  0, -0.95*ProfundidadFundamental >,
        < -0.1*FrenteFundamental, -GruesoFilo, -AnchoHojaMetal >
        translate                     AltoFundamental*y
        material { MaterialReglaCentral }
      }
      // END ReglaCentral
      // Bande Soporte
      union                             {
        difference                    {
          torus                       {
            0.01*FrenteFundamental, 0.0025*FrenteFundamental
            rotate                    90*x
          }
          plane                         {
            x, 0
          }
        }
        cylinder                        {
          0, -FrenteFundamental*x, 0.0025*FrenteFundamental
          translate             0.01*y*FrenteFundamental
        }
        translate                     AltoFundamental*y
        translate                       +0.485*FrenteFundamental*x
        translate                       -0.5*AnchoHojaMetal*z
                                        -0.0025*FrenteFundamental*z
        material                        {
          MaterialMango
        }
      }
      // END Banda Soporte
      // Seguro Hoja
      union                             {
        cylinder                        {
          -GruesoPalanca*z, 0.1*GruesoPalanca*z, 0.25*AltoFundamental
        }
        difference                      {
          torus                         {
            0.5*AltoFundamental, 0.05*AltoFundamental
            rotate                      90*x
            scale               <1/2,1,1>
            translate             ( 0.1*GruesoPalanca-0.05*AltoFundamental )*z
          }
          plane                         {
            y, 0
          }
        }
        material                        {
          MaterialMetal
        }
        translate               0.25*AltoFundamental*y
                        +( FrenteFundamental/2-AltoFundamental )*x

      }
      // END Seguro Hoja
    }
    // Calculo de R Guillotina
    #local DeltaH                       = AltoMax-AltoFundamental;
    #local DeltaX                       =
             ( FraccionDecorado+FraccionPlana-0.5 )*FrenteFundamental;
    #local LL                           = sqrt( DeltaH*DeltaH+DeltaX*DeltaX );
    #local AlfaR                        = atan2( DeltaH, DeltaX );
    #local RR                           =
                LL/sqrt( 2*( 1-cos( 2*AlfaR ) ) );
    // END Calculo de R Guillotina
    // Guillotina
    union                               {
      #local PuntoGiro                  =
        < -FrenteFundamental/2, AltoFundamental, 0 >;
      intersection                      {
        union                           {
          union                         {
            cylinder                        {
              GruesoFilo*z, GruesoPalanca*z, RR+0.6*AltoFundamental
              translate                     ( AltoFundamental+RR )*y
            }
            cylinder                        {
              0, GruesoFilo*z, RR+0.2*AltoFundamental
              translate                     ( AltoFundamental+RR )*y
            }
          }
          difference                    {
            cylinder                      {
              0, GruesoPalanca*z, AltoFundamental
              translate                   PuntoGiro
            }
            box                         {
              < -1.01*( FraccionPlana-0.5 )*FrenteFundamental, 0,
                -0.01*GruesoPalanca >,
              < 0, AltoFundamental, GruesoPalanca*1.01 >
            }
          }
        }
        sor                             {
          5,
          < RR, -GruesoPalanca >,
          < RR, -0.1*GruesoPalanca >,
          < RR, 0.7*GruesoPalanca >,
          < RR+0.3*AltoFundamental, 1.01*GruesoPalanca >,
          < RR+AltoFundamental, 1.02*GruesoPalanca >
          rotate                        90*x
          translate                     ( AltoFundamental+RR )*y
          inverse
        }
        superellipsoid                  {
          <0.2,0.6>
          scale < ( FraccionDecorado+FraccionPlana-0.5 )*FrenteFundamental,
                    4*AltoFundamental, GruesoPalanca >
        }
        union                           {
          cylinder                      {
            -GruesoPalanca*z, 2*GruesoPalanca*z,
            ( FraccionPlana+FraccionDecorado-0.5 )*FrenteFundamental
            scale         <1,8.35*AltoFundamental/FrenteFundamental,1>
          }
          box                           {
            <0,0,-GruesoPalanca>,
            <0.52*FrenteFundamental,RR,2*GruesoPalanca>
          }
        }
      }
      // Filo
      difference                      {
        cone                          {
          0, RR+0.95*AltoFundamental,
          GruesoFilo*z, RR+0.85*AltoFundamental
        }
        cylinder                      {
          -GruesoFilo*z, 2*GruesoFilo*z, RR+0.25*AltoFundamental
        }
        plane                           {
          x, -0.48*FrenteFundamental
        }
        plane                           {
          -x, -0.48*FrenteFundamental
        }
        plane                           {
          -y, 0
        }
        material                      {
          MaterialFilo
          scale                 RR
        }
        translate                     ( AltoFundamental+RR )*y
      }
      // END Filo
      // Mango
      union                             {
        union                           {
          cylinder                        {
            0, 0.01*x, 0.5
          }
          torus                           {
            0.5, 0.005
            rotate                      90*z
            translate                   +0.005*x
          }
          scale                 < LargoMango, 1.8*AltoFundamental, 1.125*GruesoPalanca >
          translate               0.52*FrenteFundamental*x+
                                1*AltoFundamental*y+0.375*GruesoPalanca*z
        }
        union                          {
          sor                           {
            7,
            < 0.8,-1 >,
            < 0.8, 0 >,
            < 0.7, 0.25 >,
            < 0.8, 0.5 >,
            < 0.7, 0.75 >,
            < 0.8, 1 >,
            < 0.8, 2 >
            rotate             -90*z
          }
          cylinder                      {
            +0.99*x, +x, 0.8
          }
          scale                 < LargoMango, 0.6*AltoFundamental, 0.5*GruesoPalanca >
          translate               0.52*FrenteFundamental*x+
                              1.2*AltoFundamental*y+0.375*GruesoPalanca*z
        }
        material                        {
          MaterialMango
        }
      }
      // END Mango
      translate                         -PuntoGiro
      rotate                            FraccionAbrir*AnguloMaximo*z
      translate                         PuntoGiro
    }
    // END Guillotina
    material                            {
      MaterialPrincipal
    }
  }
#end  // #macro GDHD_GuillotinaDiezHojasDelgadas()

#macro GDHD_Guillotina( FraccionAbrir )
  object                                {
    #local L                            = GDHD_L;
    #local FrenteFundamental            = 316*L;
    #local AltoFundamental              =  23*L;
    #local ProfundidadFundamental       = 227*L;
    #local GruesoPalanca                =  24*L;
    #local LargoMango                   =  89*L;
    #local GruesoFilo                   =   2*L;
    #local AnchoHojaMetal               =  15*L;
    #local Cuadriculado                 =  10*L;
    GDHD_GuillotinaDiezHojasDelgadas( FraccionAbrir,
        FrenteFundamental, AltoFundamental, ProfundidadFundamental,
        GruesoPalanca, LargoMango, GruesoFilo, AnchoHojaMetal,
        Cuadriculado,
        GDHD_MaterialPrincipal, GDHD_MaterialMango,
        GDHD_MaterialMetal, GDHD_MaterialFilo,
        GDHD_MaterialReglaCentral )
  }
#end  // #macro GDHD_Guillotina()
