// This file is licensed under the terms of the CC-LGPL.
//
// Autor  : Rafael Angel Campos Vargas
// Correo : RofoelCompos@hotmail.com  
// Apdo. 964-1250 Escazu, San Jose, Costa Rica
// dedicado a Jesus y Maria...
//
// En este momento estoy creando un programa
// de animacion LibreN3D para FreeDOS con el compilador FreePascal
// falta mucho... 8 noviembre 2016
//                
// Mis dibujos estan en unidades L (Lisa)
// 1 L equivale a 0.055063 pulgadas 
// definido como 0.001 para POVRAY
//     
// GDHD GuillotinaDiezHojasDelgadas
// El archivo dibuja una guillotina para papel.
// Es muy poco flexible. Es compatible con POVRay 3.1.
//
// La macro principal tiene solo un parametro. 
// #macro GDHD_Guillotina( FraccionAbrir ) 
//      FraccionAbrir           : Fraccion propia para abrir la guillotina.
//              Use 0.0 para cerrado y 1.0 brazo extendido.
//
// La siguiente macro permite mayor versatilidad.
// #macro GDHD_GuillotinaDiezHojasDelgadas( FraccionAbrir,
//                FrenteFundamental, AltoFundamental, ProfundidadFundamental,
//                GruesoPalanca, LargoMango, GruesoFilo, AnchoHojaMetal, Cuadriculado,
//                MaterialPrincipal, MaterialMango,
//                MaterialMetal, MaterialFilo, MaterialReglaCentral )
//      FraccionAbrir           : Fraccion propia para abrir la guillotina.
//              Use 0.0 para cerrado y 1.0 brazo extendido.
//      FrenteFundamental       : Referencia para el frente X del aparato.
//              Valor por defecto 316*L.
//      AltoFundamental         : Referencia para el alto Y del aparato.
//              Valor por defecto 23*L.
//      ProfundidadFundamental  : Referencia para la profundidad Z del aparato.
//              Valor por defecto 227*L.
//      GruesoPalanca           : Grueso de la palanca de la guillotina.
//              Valor por defecto 24*L.
//      LargoMango              : Largo del mango para sujetar la palanca.
//              Valor por defecto 89*L.
//      GruesoFilo              : Grueso de las laminillas de metal.
//              Valor por defecto 2*L.
//      AnchoHojaMetal          : Ancho de la lamina metalica lateral.
//              Valor por defecto 15*L.
//      Cuadriculado            : Separacion del cuadriculado.
//              Valor por defecto 10*L.
//      MaterialPrincipal       : Material principal de la guillotina.
//              Valor por defecto GDHD_MaterialPrincipal.
//      MaterialMango           : Material del mango para la palanca.
//              Valor por defecto GDHD_MAterialMango.
//      MaterialMetal           : Material para hojas metalicas.
//              Valor por defecto GDHD_MaterialMetal.
//      MaterialFilo            : Material para simular el filo de la guillotina.
//              Valor por defecto GDHD_MaterialFilo.
//      MaterialReglaCentral    : Material para la reglilla central.
//              Valor por defecto GDHD_MaterialReglaCentral.
//      
// La guillotina cerrada con parametro 0.0 se encuentra completamente contenido
// en las siguientes coordenadas:
//      Minimo  = <-165,   0,-230 >*L
//      Maximo  = < 255,  50,  30 >*L     
// Com parametro 1.0: 
//      Minimo  = <-165,   0,-230 >*L
//      Maximo  = < 160, 325,  30 >*L 
// La palanca abre hacia X+, gravedad Y-. El plano YZ separa la palanca de la base.
// La guillotina descansa enteramente sobre el plano XZ.
//
// Estas son variables predefinidas o generadas en el
// interna de las macros que no deberian modificarse directamente.
// Si quiere cambiarlas, recurra a los modificadores.  
// #declare GDHD_DefaultMaterialPrincipal       
//      Material principal de la guillotina.
// #declare GDHD_DefaultMaterialMango           
//      Material del mango para la palanca.
// #declare GDHD_DefaultMaterialMetal           
//      Material para hojas metalicas.
// #declare GDHD_DefaultMaterialFilo           
//      Material para simular el filo de la guillotina.
// #declare GDHD_DefaultMaterialReglaCentral    
//      Material para la reglilla central.
//
// Los siguientes son los modificadores
// aplicables y redefinibles por el usuario (si se indica es default)   
// #declare GDHD_ColorMarcas                = White;
//      Color de las marcas indicadoras del lateral.
// #declare GDHD_MaterialPrincipal          = material { ... }
//      Material principal de la guillotina.
// #declare GDHD_MaterialMango              = material { ... }
//      Material del mango para la palanca.
// #declare GDHD_MaterialMetal              = material { ... }
//      Material para hojas metalicas.
// #declare GDHD_MaterialFilo               = material { ... }
//      Material para simular el filo de la guillotina.
// #declare GDHD_MaterialReglaCentral       = material { ... }
//      Material para la reglilla central.
//        
// Para colocarlo simplemente
// #include "GDHD.inc" 
// object { GDHD_Guillotina(0.0) }
//                                  
// Ejemplo con la macro de texturas, mismo objeto default
// #include "GDHD.inc"   
// object                       {
//   GDHD_GuillotinaDiezHojasDelgadas( 0.0, 
//        0.316, 0.023, 0.227, 0.024, 0.089, 0.002, 0.015, 0.010,
//        GDHD_MaterialPrincipal, GDHD_MaterialMango,
//        GDHD_MaterialMetal, GDHD_MaterialFilo,
//        GDHD_MaterialReglaCentral )
// }
//        
// Con modificadores
// #include "GDHD.inc"
// #include "textures.inc"
// #declare GDHD_TexturaPrincipal   = material { textire { PinkAlabaster } }
// object        { GDHD_Guillotina(0.0) }
// 

#include "colors.inc"

#declare EjeMejorado            = true;
#declare OjoAbsoluto            = 256;
#declare VecesCentroAbsoluto    = 10;     
#declare L                      = 0.001;   
camera {
        location  <   0,  0, -( VecesCentroAbsoluto+1 )*OjoAbsoluto >*L
        direction z
        sky       y
        angle     38.74
//        look_at  <   0,  0, -VecesCentroAbsoluto*OjoAbsoluto >*L
}         
light_source    {
         < 500  , 500   , -3000 >*L
         White
}
background      {
         Gray50
}    

#include "GDHD.inc"

#local FactorAcercamiento       = 3.5;                                   

object                          {
  GDHD_Guillotina( 1.0 )
  translate       -0.130*y
  translate       -( VecesCentroAbsoluto+1 )*( FactorAcercamiento-1 )
                        /( FactorAcercamiento )*OjoAbsoluto*L*z     
}
