// This file is licensed under the terms of the CC-LGPL.
//
// Autor  : Rafael Angel Campos Vargas
// Correo : RofoelCompos@hotmail.com  
// Apdo. 964-1250 Escazu, San Jose, Costa Rica
// dedicado a Jesus y Maria...
//
// En este momento estoy creando un programa
// de animacion LibreN3D para FreeDOS con el compilador FreePascal
// falta mucho... 28 noviembre 2016
//                
// Mis dibujos estan en unidades L (Lisa)
// 1 L equivale a 0.055063 pulgadas 
// definido como 0.001 para POVRAY
//     
// PCII PinzaCocinaImportadaInoxidable 
// El archivo dibuja una pinza para cocina.
// Es muy poco flexible. Es compatible con POVRay 3.1.
//
// La macro principal tiene tres parametros. Tomar en cuenta que la pinza no se dibuja
//      bien si utiliza todos los parametros nulos. Para cerrar completamente podria
//      utilizar los valores (0.0,1.0,0.3). 
// #macro PCII_Pinza( FraccionAbierto, FraccionSeguro, FraccionPaleta ) 
//      FraccionAbierto         : Fraccion para abrir la pinza, 0.0 para cerrado y
//              1.0 para completamente abierto.
//      FraccionSeguro          : La pinza original se asegura al desplazar
//              un poco la estructura principal hacia abajo y cerrar la boca.
//              Este parametro simula este desplazamiento, desde 0.0 hasta 1.0.
//      FraccionPaleta          : Dobla la pinza hacia adentro. Utilice 0.0 para
//              no doblar, y 1.0 para el maximo.
//
// La siguiente macro permite mayor versatilidad.
// #macro PCII_PinzaCocinaImportadaInoxidable(
//                FraccionAbierto, FraccionSeguro, FraccionPaleta,
//                AnguloAbierto, AnguloPaleta, LargoFundamental,
//                GruesoXFundamental, GruesoZFundamental,
//                GruesoMetal, EjeGiro, RadioOjo, 
//                LargoColgar, LargoAgarre, AnchoAgarre,
//                MaterialMetalico, MaterialAgarre )
//      FraccionAbierto         : Fraccion para abrir la pinza, 0.0 para cerrado y
//              1.0 para completamente abierto.
//      FraccionSeguro          : La pinza original se asegura desplazando
//              un poco la estructura principal hacia abajo y cerrando la boca.
//              Este parametro simula este desplazamiento, desde 0.0 hasta 1.0.
//      FraccionPaleta          : Dobla la pinza hacia adentro. Utilice 0.0 para
//              no doblar, y 1.0 para el maximo.
//      AnguloAbierto           : Angulo maximo de apertura de la pinza.
//              Valor por defecto 30.
//      AnguloPaleta            : Angulo maximo de cierre de las paletas.
//              Valor por defecto 17.5.
//      LargoFundamental        : Referencia para el largo Y de las pinzas.
//              Valor por defecto 145*L.
//      GruesoXFundamental      : Referencia para el grueso X de las pinzas.
//              Valor por defecto 6.35*L.
//      GruesoZFundamental      : Referencia para el grueso Z de las pinzas.
//              Valor por defecto 11.8*L.
//      GruesoMetal             : Grueso principal del metal utilizado.
//              Valor por defecto 1*L.
//      EjeGiro                 : Referencia para ubicar en Y el eje de giro.
//              Valor por defecto 9*L.
//      RadioOjo                : Radio del anillo para colgar la pinza.
//              Valor por defecto 6.25*L.
//      LargoColgar             : Aplica la base fija no movible con la pinza.
//              Valor por defecto 21.5*L.
//      LargoAgarre             : Largo de la paleta de agarre.
//              Valor por defecto 56.3*L.
//      AnchoAgarre             : Ancho en Z de la paleta de agarre.
//              Valor por defecto 40*L.                      
//      MaterialMetalico        : Material principal a aplicar al aparato.
//              Valor por defecto PCII_MaterialMetalico.
//      MaterialAgarre          : Material para la paleta de agarre.
//              Valor por defecto PCII_MaterialAgarre.
//
// La pinza completamente abierta con parametros (1,0,1) se ubica completamente
// en las siguientes coordenadas:      
//      Minimo  = < -43.5,-210.0, -25.5 >*L
//      Maximo  = <  43.5,   8.0,  25.5 >*L
// El anillo para colgar la pinza se encuentra centrado en el plano YZ. Cuelga
// con gravedad Y-, con los dos brazos a ambos lados del eje Y.     
//
// Estas son variables predefinidas o generadas en el
// interna de las macros que no deberian modificarse directamente.
// Si quiere cambiarlas, recurra a los modificadores.  
// #declare PCII_DefaultMaterialMetalico      
//      Material principal a aplicar al aparato.
// #declare PCII_DefaultMaterialAgarre           
//      Material para la paleta de agarre.
//
// Los siguientes son los modificadores
// aplicables y redefinibles por el usuario (si se indica es default)   
// #declare PCII_MaterialMetalico           = material { ... }
//      Material principal a aplicar al aparato.
// #declare PCII_MaterialAgarre             = material { ... }
//      Material para la paleta de agarre.
//        
// Para colocarlo simplemente
// #include "PCII.inc" 
// object { PCII_Pinza(1.0,0.0,1.0) }
//                                  
// Ejemplo con la macro de texturas, mismo objeto default
// #include "PCII.inc"   
// object                       {
//   PCII_PinzaCocinaImportadaInoxidable( 1.0, 0.0, 1.0,
//         30, 17.5, 0.145, 0.00635, 0.0118, 0.001, 0.009,
//         0.00625, 0.0215, 0.0563, 0.040,           
//         PCII_MaterialMetalico, PCII_MaterialAgarre )
// }
//        
// Con modificadores
// #include "PCII.inc"
// #include "textures.inc"
// #declare PCII_MaterialMetalico   = material { textire { PinkAlabaster } }
// object        { PCII_Pinza(1.0,0.0,1.0) }
// 
                          
#declare PCII_L                         = 0.001;

#declare PCII_DefaultMaterialMetalico   =
  material                              {
    texture                             {
      pigment                           {
        Silver*0.6
      }
      normal                            {
        bumps
      }
      finish                            {
        metallic
        phong  0.5
        phong_size      20
      }
    }
  }

#ifndef ( PCII_MaterialMetalico )
  #declare PCII_MaterialMetalico        = PCII_DefaultMaterialMetalico
#end  // #ifndef ( PCII_MaterialMetalico )

#declare PCII_DefaultMaterialAgarre     =
  material                              {
    texture                             {
      pigment                           {
        Gray30
      }
    }
  }

#ifndef ( PCII_MaterialAgarre )
  #declare PCII_MaterialAgarre          = PCII_DefaultMaterialAgarre
#end  // #ifndef ( PCII_MaterialAgarre )

#macro PCII_PinzaCocinaImportadaInoxidable(
                FraccionAbierto, FraccionSeguro, FraccionPaleta,
                AnguloAbierto, AnguloPaleta, LargoFundamental,
                GruesoXFundamental, GruesoZFundamental,
                GruesoMetal, EjeGiro, RadioOjo, 
                LargoColgar, LargoAgarre, AnchoAgarre,
                MaterialMetalico, MaterialAgarre )
  union                                 {
    // Ojo FIJO
    union                               {
      #local RadioTorusMenor            = 0.37*RadioOjo;
      torus                             {
        RadioOjo, RadioTorusMenor
        rotate                          90*x
        material                        {
          MaterialAgarre
        }
      }
      #local EscaleAgarre       =  < ( LargoColgar+RadioOjo-RadioTorusMenor )*
                        tan( radians( AnguloAbierto/2 ) ),
                        ( LargoColgar+RadioOjo-RadioTorusMenor ),
                          GruesoMetal/2>;
      difference                        {
        union                           {
          difference                    {
            box                         {
              -1, 1
              rotate                        45*z
              scale                     EscaleAgarre/< sqrt(2), sqrt(2), 1 >
            }
            plane                       {
              y, 0
            }
            translate           -LargoColgar*y
          }
          box                           {
            < -EscaleAgarre.x, -LargoColgar, -GruesoMetal/2 >,
            <  EscaleAgarre.x, -LargoColgar-RadioOjo, GruesoMetal/2 >
          }
        }
        cylinder                        {
          -RadioOjo*z, RadioOjo*z, RadioOjo
        }
        material                        {
          MaterialMetalico
        }
      }
    }
    // END Ojo FIJO
    // Brazo
    #local C                          = 0;
    #while ( C < 2 )
      union                             {
        // BrazoMetalico
        union                           {
          difference                    {
            union                       {
              superellipsoid              {
                < 0.1, 0.1 >
                scale     < GruesoXFundamental, LargoFundamental,
                                  GruesoZFundamental >/2
                translate                   -LargoFundamental/2*y
              }
            }
            box                         {
              -1, 1
              scale     < GruesoXFundamental, 1.1*LargoFundamental,
                                0.8*GruesoZFundamental >/2
              translate         -GruesoXFundamental*x/2
              translate                   -LargoFundamental/2*y
            }
            superellipsoid              {
              < 0.1, 0.1 >
              scale     < GruesoXFundamental, 1.1*LargoFundamental,
                                GruesoZFundamental >/2-GruesoMetal*<1,0,1>
              translate                   -LargoFundamental/2*y
            }
          }
          difference                    {
            prism                         {
              cubic_spline
              -GruesoZFundamental/2, +GruesoZFundamental/2,
              8,
              < +GruesoZFundamental/8, -2*EjeGiro >,
              < +GruesoZFundamental/8, 0 >,
              < -EscaleAgarre.x, 0 >,
              < -EscaleAgarre.x-RadioOjo/2, -EjeGiro >,
              < -EscaleAgarre.x, -2*EjeGiro >,
              < +GruesoZFundamental/8, -2*EjeGiro >,
              < +GruesoZFundamental/8, 0 >,
              < -EscaleAgarre.x, 0 >
              rotate                      -90*x
            }
            box                         {
              -1, 1
              scale     < LargoFundamental, LargoFundamental,
                                GruesoZFundamental/2-GruesoMetal >
            }
          }
          // Cuero Lateral
          difference                    {
            superellipsoid              {
              < 0.5, 0.5 >
              scale     < GruesoXFundamental/3, 2*LargoFundamental/3,
                                GruesoZFundamental/2-3*GruesoMetal >
            }
            plane                       {
              -y, 0.025*LargoFundamental
            }
            material                    {
              MaterialAgarre
            }
            translate                   +GruesoXFundamental/2*x
          }
          // END Cuero Lateral
          // Pinza
          union                         {
            // Union Pinza
            superellipsoid              {
              <0.1,0.1>
              scale             < GruesoXFundamental/2-GruesoMetal,
                                  GruesoXFundamental/2,
                                  GruesoZFundamental/2-GruesoMetal >
              translate                 -LargoFundamental*y
            }
            // END Union Pinza
            difference                  {
              #local AnguloBorde        =
                        degrees( atan2( 0.5*AnchoAgarre, LargoAgarre ) );
              intersection              {
                prism                     {
                  cubic_spline
                  -GruesoXFundamental/2, +GruesoXFundamental/2,
                  11,
                  < 0, +GruesoZFundamental-GruesoMetal >,
                  <-GruesoXFundamental/2, 0 >,
                  < 0, -GruesoZFundamental+GruesoMetal >,
                  < 0.15*LargoAgarre, -0.7*AnchoAgarre/2 >,
                  < 0.85*LargoAgarre, -AnchoAgarre/2 >,
                  < LargoAgarre, 0 >,
                  < 0.85*LargoAgarre, +AnchoAgarre/2 >,
                  < 0.15*LargoAgarre, +0.7*AnchoAgarre/2 >,
                  < 0, +GruesoZFundamental-GruesoMetal >,
                  <-GruesoXFundamental/2, 0 >,
                  < 0, -GruesoZFundamental+GruesoMetal >
                  rotate                        -90*z
                }
                box                             {
                  -1, 1
                  rotate        45*z
                  scale < GruesoXFundamental/3,
                        1.2*LargoAgarre, LargoFundamental+AnchoAgarre >/
                        <sqrt(2),sqrt(2),1>
                }
              }
              #local D                          = -1;
              #while ( D < 2 )
                union                           {
                  #local AnchoPerforado         = 0.10*AnchoAgarre;
                  #local LargoPerforado         = 0.65*LargoAgarre;
                  box                           {
                    <-GruesoXFundamental,0,-AnchoPerforado/2>,
                    <+GruesoXFundamental,-LargoPerforado,+AnchoPerforado/2>
                  }
                  cylinder                      {
                    -GruesoXFundamental*x, GruesoXFundamental*x,
                    AnchoPerforado/2
                  }
                  cylinder                      {
                    -GruesoXFundamental*x, GruesoXFundamental*x,
                    AnchoPerforado/2
                    translate           -LargoPerforado*y
                  }
                  rotate                -0.25*D*AnguloBorde*x
                  translate      -0.2*LargoAgarre*y+D*AnchoAgarre*z*0.2
                }
                #local D                        = D+1;
              #end  // #while ( C < 2 )
              rotate            -FraccionPaleta*AnguloPaleta*z
              translate     -( LargoFundamental+GruesoXFundamental/2 )*y
            }
            material                    {
              MaterialAgarre
            }
          }
          // END Pinza
          material                      {
            MaterialMetalico
          }
        }
        #local DeltaDescender           = LargoColgar
                -2*( RadioOjo+RadioTorusMenor );
        translate               -( GruesoXFundamental/2-2*GruesoMetal )*x
                                -( GruesoZFundamental/2-GruesoMetal )*z
        translate       -( 2*RadioOjo+RadioTorusMenor )*y
                        /cos( radians( AnguloAbierto/2 ) )
        rotate          AnguloAbierto*z/2
        translate       +( RadioOjo-RadioTorusMenor )*y
                        -FraccionSeguro*DeltaDescender*y
        #local TornilloGiro             = EjeGiro+
                FraccionSeguro*DeltaDescender+RadioOjo+2*RadioTorusMenor;
        translate               +TornilloGiro*y
        rotate                  ( -( 1-FraccionAbierto )*AnguloAbierto/2 )*z
        translate               -TornilloGiro*y
        // END BrazoMetalico
        #if ( C = 1 )
          translate                     1.5*GruesoMetal*z
          scale                         <-1,1,1>
        #end  // #if ( C = 1 )
      }
      #local C                        = C+1;
    #end  // #while ( C < 2 )
    // END Brazo
    // Tornillo Giro
    union                               {
      sphere                            {
        0, RadioOjo/2
        scale                   <1,1,1/2>
        translate               -( GruesoZFundamental/2+4*GruesoMetal )*z
                                -TornilloGiro*y
      }
      difference                        {
        cylinder                        {
          -( GruesoZFundamental/2+4*GruesoMetal )*z,
          +3*GruesoMetal*z, RadioOjo/2
        }
        cylinder                        {
          -( GruesoZFundamental/2+5*GruesoMetal )*z,
          +4*GruesoMetal*z, RadioOjo/2-0.1*GruesoMetal
        }
        translate                       -TornilloGiro*y
      }
      torus                             {
        3*RadioOjo/8, RadioOjo/8
        rotate                  90*x
        scale                   <1,1,1/2>
        translate               +( 3*GruesoMetal )*z
                                -TornilloGiro*y
      }
      // END TornilloGiro
      material                          {
        MaterialMetalico
      }
    }
    material                            {
      MaterialAgarre
    }
  }
#end  // #macro PCII_PinzaCocinaImportadaInoxidable

#macro PCII_Pinza( FraccionAbierto, FraccionSeguro, FraccionPaleta )
  object                                {
    #local L                            = PCII_L;
    #local AnguloAbierto                = 30;
    #local AnguloPaleta                 = 17.5;
    #local LargoFundamental             = 145*L;
    #local GruesoXFundamental           =   6.35*L;
    #local GruesoZFundamental           =  11.8*L;
    #local GruesoMetal                  =   1*L;
    #local EjeGiro                      =   9*L;
    #local RadioOjo                     =   6.25*L;
    #local LargoColgar                  =  21.5*L;
    #local LargoAgarre                  =  56.3*L;
    #local AnchoAgarre                  =  40*L;
    // Separacion entre en centro del ojo y la pinza propiamente
    // en estado cerrado
    PCII_PinzaCocinaImportadaInoxidable(
         FraccionAbierto, FraccionSeguro, FraccionPaleta,
         AnguloAbierto, AnguloPaleta, LargoFundamental,
         GruesoXFundamental, GruesoZFundamental,
         GruesoMetal, EjeGiro, RadioOjo, 
         LargoColgar, LargoAgarre, AnchoAgarre,
         PCII_MaterialMetalico, PCII_MaterialAgarre )
  }
#end  // PCII_Pinza()
