// This file is licensed under the terms of the CC-LGPL.
//
// Autor  : Rafael Angel Campos Vargas
// Correo : RofoelCompos@hotmail.com  
// Apdo. 964-1250 Escazu, San Jose, Costa Rica
// dedicado a Jesus y Maria...
//
// En este momento estoy creando un programa
// de animacion LibreN3D para FreeDOS con el compilador FreePascal
// falta mucho... 4 octubre 2016
//                
// Mis dibujos estan en unidades L (Lisa)
// 1 L equivale a 0.055063 pulgadas 
// definido como 0.001 para POVRAY
//      
// PCMC PlanchaCuidadoMaterialCaliente
// El archivo dibuja una plancha algo pasada de moda para uso domstico, 
// es muy poco flexible pero incluye una macro para modificar
// texturas y unas pocas caracteristicas. Es compatible con POV-Ray 3.1.
//
// La macro principal tiene dos parametros.   
// #macro PCMC_Plancha( AnguloNivel, BanderaLuz )   
//      AnguloNivel             : Gira la barrita para graduar la
//              temperatura un angulo en grados, positivo o negativo.
//      BanderaLuz              : "on" simula el encendido de una
//              luz indicadora. "off" la luz se encuentra apagada.               
//
// La siguiente macro permite mayor control de las caracteristicas.
// #macro PCMC_PlanchaCuidadoMaterialCaliente( AnguloNivel, BanderaLuz,
//                LargoBase, GruesoBaseCaliente, AltoMetalico,
//                AnguloAgarradera, AnchoBase, ExcesoLargoBase,
//                MaterialPrincipal, MaterialMetalico,
//                MaterialMetalicoCaliente, MaterialBaseAgarradera )
//      AnguloNivel             : Gira la barrita para graduar la
//              temperatura un angulo en grados, positivo o negativo.
//      BanderaLuz              : "on" simula el encendido de una
//              luz indicadora. "off" la luz se encuentra apagada. 
//      LargoBase               : Referencia para el largo en X de la plancha.
//              Valor por defecto 140*L.
//      GruesoBaseCaliente      : Grueso de la placa metalica en la base caliente.
//              Valor por defecto 7*L.
//      AltoMetalico            : Referencia para el alto de la base metalica principal
//              de la plancha. Valor por defecto 20*L.
//      AnguloAgarradera        : Inclinacion para la estructura superior de la plancha.
//              Valor por defecto 2.86.
//      AnchoBase               : Referencia para el ancho Z de la plancha.
//              Valor por defecto 84*L.
//      ExcesoLargoBase         : Referencia para el largo en X adicional total de la plancha
//              respecto a su base caliente. Valor por defecto 25*L.
//      MaterialPrincipal       : Material a aplicar a la estructura de la agarradera.
//              Valor por defecto PCMC_MaterialPrincipal.
//      MaterialMetalico        : Material principal para los elementos metalicos.
//              Valor por defecto PCMC_MaterialMetalico.
//      MaterialMetalicoCaliente        : Material para la base caliente inferior.
//              Valor por defecto PCMC_MaterialMetalicoCaliente.
//      MaterialBaseAgarradera          : Material a aplicar a la base de la agarradera.
//              Valor por defecto PCMC_MaterialBaseAgarradera.             
//      
// La plancha con sus valores por defecto se encuentra completamente contenida en:   
//      Punto Minimo    = <-95,  0,-45>*L
//      Punto Maximo    = < 73, 76, 45>*L 
// La plancha mira con el frente hacia X+ gravedad Y-, descansa parcialmente 
// centrada sobre el origen del plano XZ.
//
// Estas son variables predefinidas o generadas en el
// interior de las macros que no deberian modificarse directamente.
// Si quiere cambiarlas, recurra a los modificadores.  
// #declare PCMC_DefaultMaterialPrincipal       
//      Material a aplicar a la estructura de la agarradera.
// #declare PCMC_DefaultMaterialMetalico        
//      Material principal para los elementos metalicos.
// #declare PCMC_DefaultMaterialMetalicoCaliente        
//      Material para la base caliente inferior.
// #declare PCMC_DefaultMaterialBaseAgarradera          
//      Material a aplicar a la base de la agarradera.
//
// Los siguientes son los modificadores
// aplicables y redefinibles por el usuario (si se indica es default)   
// #declare PCMC_MaterialPrincipal                  = material { ... }      
//      Material a aplicar a la estructura de la agarradera.
// #declare PCMC_MaterialMetalico                   = material { ... } 
//      Material principal para los elementos metalicos.
// #declare PCMC_MaterialMetalicoCaliente           = material { ... } 
//      Material para la base caliente inferior.
// #declare PCMC_MaterialBaseAgarradera             = material { ... } 
//      Material a aplicar a la base de la agarradera.
// #declare PCMC_ColorNivel                         = Orange;
//      Color a aplicar a la barrita selectora de temperatura.
//        
// Para colocarlo simplemente
// #include "PCMC.inc" 
// object { PCMC_Plancha( 0, off ) }
//                                 
// Ejemplo con la macro detallada, mismo objeto default
// #include "PCMC.inc"   
// object                       {
//   PCMC_PlanchaCuidadoMaterialCaliente( 0, off, 0.140, 0.007, 0.020, 2.86, 0.084, 0.025,
//        PCMC_MaterialPrincipal, PCMC_MaterialMetalico,
//        PCMC_MaterialMetalicoCaliente, PCMC_MaterialBaseAgarradera )  
// }
//        
// Con modificadores
// #include "PCMC.inc"
// #include "textures.inc"
// #declare PCMC_MaterialPrincipal   = material { textures { PinkAlabaster } }
// object        { PCMC_Plancha( 0, off ) }
//
#if ( version >= 3.7 )
  #version 3.7; 
#end

global_settings { assumed_gamma 1.0 }

#include "colors.inc"

#declare OjoAbsoluto            = 256;
#declare VecesCentroAbsoluto    = 10;     
#declare L                      = 0.001;
camera {
        location  <   0,  0, -( VecesCentroAbsoluto+1 )*OjoAbsoluto >*L
        direction z
        sky       y
        angle     38.74
//        look_at  <   0,  0, -VecesCentroAbsoluto*OjoAbsoluto >*L
}         
light_source    {
         < 500  , 500   , -3000 >*L
         White
}
background      {
         Gray50
}    

#include "PCMC.inc"

#declare FactorAcercamiento             = 10;
  
object                                  {
  PCMC_Plancha( 30, off )   
  translate         +0.015*x-0.035*y
  translate       -( VecesCentroAbsoluto+1 )*( FactorAcercamiento-1 )
                        /( FactorAcercamiento )*OjoAbsoluto*L*z      
}
